@@ -18,8 +18,6 @@
 #include <log4cxx/logstring.h>
 #include <log4cxx/helpers/filewatchdog.h>
 #include <log4cxx/helpers/loglog.h>
-#include <apr_thread_proc.h>
-#include <apr_atomic.h>
 #include <log4cxx/helpers/transcoder.h>
 #include <log4cxx/helpers/exception.h>
 #include <log4cxx/helpers/threadutility.h>
@@ -27,16 +25,23 @@
 #include <functional>
 #include <chrono>
 
-using namespace log4cxx;
-using namespace log4cxx::helpers;
+#if LOG4CXX_EVENTS_AT_EXIT
+#include <log4cxx/private/atexitregistry.h>
+#endif
+
+using namespace LOG4CXX_NS;
+using namespace LOG4CXX_NS::helpers;
 
 long FileWatchdog::DEFAULT_DELAY = 60000;
 
 struct FileWatchdog::FileWatchdogPrivate{
 	FileWatchdogPrivate(const File& file1) :
 		file(file1), delay(DEFAULT_DELAY), lastModif(0),
-		warnedAlready(false), interrupted(0), thread(){}
-
+		warnedAlready(false), interrupted(0), thread()
+#if LOG4CXX_EVENTS_AT_EXIT
+		, atExitRegistryRaii([this]{stopWatcher();})
+#endif
+        { stopWatcher(); }
 	/**
 	The name of the file to observe  for changes.
 	*/
@@ -53,6 +58,21 @@ struct FileWatchdog::FileWatchdogPrivate{
 	std::thread thread;
 	std::condition_variable interrupt;
 	std::mutex interrupt_mutex;
+
+#if LOG4CXX_EVENTS_AT_EXIT
+	helpers::AtExitRegistry::Raii atExitRegistryRaii;
+#endif
+
+	void stopWatcher()
+	{
+        {
+            std::lock_guard<std::mutex> lock(interrupt_mutex);
+            interrupted = 0xFFFF;
+        }
+        interrupt.notify_all();
+        if (thread.joinable())
+            thread.join();
+	}
 };
 
 FileWatchdog::FileWatchdog(const File& file1)
@@ -62,28 +82,43 @@ FileWatchdog::FileWatchdog(const File& file1)
 
 FileWatchdog::~FileWatchdog()
 {
-	m_priv->interrupted = 0xFFFF;
-
-	{
-		std::unique_lock<std::mutex> lock(m_priv->interrupt_mutex);
-		m_priv->interrupt.notify_all();
-	}
-	m_priv->thread.join();
+	if (m_priv->thread.joinable())
+		stop();
 }
 
-const File& FileWatchdog::file(){
+
+bool FileWatchdog::is_active()
+{
+	return m_priv->thread.joinable();
+}
+
+void FileWatchdog::stop()
+{
+	LogLog::debug(LOG4CXX_STR("Stopping file watchdog"));
+	m_priv->stopWatcher();
+}
+
+const File& FileWatchdog::file()
+{
 	return m_priv->file;
 }
 
 void FileWatchdog::checkAndConfigure()
 {
+	if (LogLog::isDebugEnabled())
+	{
+		LogString msg(LOG4CXX_STR("Checking ["));
+		msg += m_priv->file.getPath();
+		msg += LOG4CXX_STR("]");
+		LogLog::debug(msg);
+	}
 	Pool pool1;
 
 	if (!m_priv->file.exists(pool1))
 	{
 		if (!m_priv->warnedAlready)
 		{
-			LogLog::debug(((LogString) LOG4CXX_STR("["))
+			LogLog::warn(LOG4CXX_STR("[")
 				+ m_priv->file.getPath()
 				+ LOG4CXX_STR("] does not exist."));
 			m_priv->warnedAlready = true;
@@ -91,7 +126,7 @@ void FileWatchdog::checkAndConfigure()
 	}
 	else
 	{
-		apr_time_t thisMod = m_priv->file.lastModified(pool1);
+		auto thisMod = m_priv->file.lastModified(pool1);
 
 		if (thisMod > m_priv->lastModif)
 		{
@@ -104,33 +139,41 @@ void FileWatchdog::checkAndConfigure()
 
 void FileWatchdog::run()
 {
-	LogString msg(LOG4CXX_STR("Checking ["));
-	msg += m_priv->file.getPath();
-	msg += LOG4CXX_STR("] at ");
-	StringHelper::toString((int)m_priv->delay, m_priv->pool, msg);
-	msg += LOG4CXX_STR(" ms interval");
-	LogLog::debug(msg);
-
-	while (m_priv->interrupted != 0xFFFF)
+	if (LogLog::isDebugEnabled())
 	{
-		std::unique_lock<std::mutex> lock( m_priv->interrupt_mutex );
-		m_priv->interrupt.wait_for( lock, std::chrono::milliseconds( m_priv->delay ),
-			std::bind(&FileWatchdog::is_interrupted, this) );
-
-		checkAndConfigure();
+		LogString msg(LOG4CXX_STR("Checking ["));
+		msg += m_priv->file.getPath();
+		msg += LOG4CXX_STR("] at ");
+		StringHelper::toString((int)m_priv->delay, m_priv->pool, msg);
+		msg += LOG4CXX_STR(" ms interval");
+		LogLog::debug(msg);
 	}
 
-	LogString msg2(LOG4CXX_STR("Stop checking ["));
-	msg2 += m_priv->file.getPath();
-	msg2 += LOG4CXX_STR("]");
-	LogLog::debug(msg2);
+	while (!is_interrupted())
+	{
+		std::unique_lock<std::mutex> lock( m_priv->interrupt_mutex );
+		if (!m_priv->interrupt.wait_for( lock, std::chrono::milliseconds( m_priv->delay ),
+			std::bind(&FileWatchdog::is_interrupted, this) ))
+			checkAndConfigure();
+	}
+
+	if (LogLog::isDebugEnabled())
+	{
+		LogString msg2(LOG4CXX_STR("Stop checking ["));
+		msg2 += m_priv->file.getPath();
+		msg2 += LOG4CXX_STR("]");
+		LogLog::debug(msg2);
+	}
 }
 
 void FileWatchdog::start()
 {
 	checkAndConfigure();
-
-	m_priv->thread = ThreadUtility::instance()->createThread( LOG4CXX_STR("FileWatchdog"), &FileWatchdog::run, this );
+	if (!m_priv->thread.joinable())
+	{
+		m_priv->interrupted = 0;
+		m_priv->thread = ThreadUtility::instance()->createThread(LOG4CXX_STR("FileWatchdog"), &FileWatchdog::run, this);
+	}
 }
 
 bool FileWatchdog::is_interrupted()
