@@ -313,10 +313,13 @@ LevelPtr OptionConverter::toLevel(const LogString& value,
 		}
 		else
 		{
-			LogLog::debug(
-				((LogString) LOG4CXX_STR("OptionConverter::toLevel: no class name specified, level=["))
-				+ value
-				+ LOG4CXX_STR("]"));
+			if (LogLog::isDebugEnabled())
+			{
+				LogLog::debug(
+					((LogString) LOG4CXX_STR("OptionConverter::toLevel: no class name specified, level=["))
+					+ value
+					+ LOG4CXX_STR("]"));
+			}
 			// no class name specified : use standard Level class
 			return Level::toLevelLS(value, defaultValue);
 		}
@@ -324,8 +327,13 @@ LevelPtr OptionConverter::toLevel(const LogString& value,
 
 	LogString clazz = value.substr(hashIndex + 1);
 	LogString levelName = value.substr(0, hashIndex);
-	LogLog::debug(((LogString) LOG4CXX_STR("OptionConverter::toLevel: class=["))
-		+ clazz + LOG4CXX_STR("], level=[") + levelName + LOG4CXX_STR("]"));
+	if (LogLog::isDebugEnabled())
+	{
+		LogLog::debug(LOG4CXX_STR("OptionConverter::toLevel: class=[")
+		+ clazz + LOG4CXX_STR("], level=[")
+		+ levelName + LOG4CXX_STR("]")
+		);
+	}
 
 	// This is degenerate case but you never know.
 	if (levelName.empty())
@@ -427,7 +435,8 @@ void OptionConverter::selectAndConfigure(const File& configFileName,
 
 	if (!clazz.empty())
 	{
-		LogLog::debug(LOG4CXX_STR("Preferred configurator class: ") + clazz);
+		if (LogLog::isDebugEnabled())
+			LogLog::debug(LOG4CXX_STR("Preferred configurator class: ") + clazz);
 		const Class& clazzObj = Loader::loadClass(clazz);
 		ObjectPtr obj = ObjectPtr(clazzObj.newInstance());
 		configurator = LOG4CXX_NS::cast<Configurator>(obj);
