@@ -19,18 +19,25 @@
 #include <log4cxx/helpers/loglog.h>
 #include <log4cxx/helpers/optionconverter.h>
 #include <log4cxx/helpers/stringhelper.h>
-#include <apr_thread_proc.h>
-#include <apr_atomic.h>
-#include <apr_strings.h>
+#include <log4cxx/helpers/serversocket.h>
 #include <log4cxx/helpers/charsetencoder.h>
 #include <log4cxx/helpers/bytebuffer.h>
 #include <log4cxx/helpers/threadutility.h>
 #include <log4cxx/private/appenderskeleton_priv.h>
 #include <mutex>
+#include <thread>
+#include <vector>
 
-using namespace log4cxx;
-using namespace log4cxx::helpers;
-using namespace log4cxx::net;
+#if LOG4CXX_EVENTS_AT_EXIT
+#include <log4cxx/private/atexitregistry.h>
+#endif
+
+using namespace LOG4CXX_NS;
+using namespace LOG4CXX_NS::helpers;
+using namespace LOG4CXX_NS::net;
+
+typedef helpers::SocketPtr Connection;
+LOG4CXX_LIST_DEF(ConnectionList, Connection);
 
 IMPLEMENT_LOG4CXX_OBJECT(TelnetAppender)
 
@@ -41,16 +48,43 @@ struct TelnetAppender::TelnetAppenderPriv : public AppenderSkeletonPrivate
 		connections(maxConnections),
 		encoding(LOG4CXX_STR("UTF-8")),
 		encoder(CharsetEncoder::getUTF8Encoder()),
-		sh(),
-		activeConnections(0) {}
+		activeConnections(0)
+#if LOG4CXX_EVENTS_AT_EXIT
+		, atExitRegistryRaii([this]{stopAcceptingConnections();})
+#endif
+        { stopAcceptingConnections(); }
 
 	int port;
 	ConnectionList connections;
 	LogString encoding;
-	log4cxx::helpers::CharsetEncoderPtr encoder;
+	LOG4CXX_NS::helpers::CharsetEncoderPtr encoder;
 	std::unique_ptr<helpers::ServerSocket> serverSocket;
 	std::thread sh;
 	size_t activeConnections;
+
+#if LOG4CXX_EVENTS_AT_EXIT
+	helpers::AtExitRegistry::Raii atExitRegistryRaii;
+#endif
+
+	void stopAcceptingConnections()
+	{
+		{
+			std::lock_guard<std::recursive_mutex> lock(this->mutex);
+			if (!this->serverSocket || this->closed)
+				return;
+			this->closed = true;
+		}
+		// Interrupt accept()
+		try
+		{
+			this->serverSocket->close();
+		}
+		catch (Exception&)
+		{
+		}
+		if (this->sh.joinable())
+			this->sh.join();
+	}
 };
 
 #define _priv static_cast<TelnetAppenderPriv*>(m_priv.get())
@@ -79,7 +113,8 @@ void TelnetAppender::activateOptions(Pool& /* p */)
 		_priv->serverSocket->setSoTimeout(1000);
 	}
 
-	_priv->sh = ThreadUtility::instance()->createThread( LOG4CXX_STR("TelnetAppender"), &TelnetAppender::acceptConnections, this );
+	if (!_priv->sh.joinable())
+		_priv->sh = ThreadUtility::instance()->createThread( LOG4CXX_STR("TelnetAppender"), &TelnetAppender::acceptConnections, this );
 }
 
 void TelnetAppender::setOption(const LogString& option,
@@ -89,6 +124,10 @@ void TelnetAppender::setOption(const LogString& option,
 	{
 		setPort(OptionConverter::toInt(value, DEFAULT_PORT));
 	}
+	else if (StringHelper::equalsIgnoreCase(option, LOG4CXX_STR("MAXCONNECTIONS"), LOG4CXX_STR("maxconnections")))
+	{
+		setMaxConnections(OptionConverter::toInt(value, MAX_CONNECTIONS));
+	}
 	else if (StringHelper::equalsIgnoreCase(option, LOG4CXX_STR("ENCODING"), LOG4CXX_STR("encoding")))
 	{
 		setEncoding(value);
@@ -115,65 +154,36 @@ void TelnetAppender::setEncoding(const LogString& value)
 
 void TelnetAppender::close()
 {
+	_priv->stopAcceptingConnections();
 	std::lock_guard<std::recursive_mutex> lock(_priv->mutex);
-
-	if (_priv->closed)
-	{
-		return;
-	}
-
-	_priv->closed = true;
-
 	SocketPtr nullSocket;
-
-	for (ConnectionList::iterator iter = _priv->connections.begin();
-		iter != _priv->connections.end();
-		iter++)
+	for (auto& item : _priv->connections)
 	{
-		if (*iter != 0)
+		if (item)
 		{
-			(*iter)->close();
-			*iter = nullSocket;
+			item->close();
+			item = nullSocket;
 		}
 	}
-
-	if (_priv->serverSocket != NULL)
-	{
-		try
-		{
-			_priv->serverSocket->close();
-		}
-		catch (Exception&)
-		{
-		}
-	}
-
-	if ( _priv->sh.joinable() )
-	{
-		_priv->sh.join();
-	}
-
 	_priv->activeConnections = 0;
 }
 
 
 void TelnetAppender::write(ByteBuffer& buf)
 {
-	for (ConnectionList::iterator iter = _priv->connections.begin();
-		iter != _priv->connections.end();
-		iter++)
+	for (auto& item :_priv->connections)
 	{
-		if (*iter != 0)
+		if (item)
 		{
 			try
 			{
 				ByteBuffer b(buf.current(), buf.remaining());
-				(*iter)->write(b);
+				item->write(b);
 			}
 			catch (Exception&)
 			{
 				// The client has closed the connection, remove it from our list:
-				*iter = 0;
+				item.reset();
 				_priv->activeConnections--;
 			}
 		}
@@ -204,7 +214,10 @@ void TelnetAppender::append(const spi::LoggingEventPtr& event, Pool& p)
 	if (count > 0)
 	{
 		LogString msg;
-		_priv->layout->format(msg, event, _priv->pool);
+		if (_priv->layout)
+			_priv->layout->format(msg, event, p);
+		else
+			msg = event->getMessage();
 		msg.append(LOG4CXX_STR("\r\n"));
 		size_t bytesSize = msg.size() * 2;
 		char* bytes = p.pstralloc(bytesSize);
@@ -270,13 +283,11 @@ void TelnetAppender::acceptConnections()
 				//
 				std::lock_guard<std::recursive_mutex> lock(_priv->mutex);
 
-				for (ConnectionList::iterator iter = _priv->connections.begin();
-					iter != _priv->connections.end();
-					iter++)
+				for (auto& item : _priv->connections)
 				{
-					if (*iter == NULL)
+					if (!item)
 					{
-						*iter = newClient;
+						item = newClient;
 						_priv->activeConnections++;
 
 						break;
@@ -321,3 +332,30 @@ void TelnetAppender::setPort(int port1)
 {
 	_priv->port = port1;
 }
+
+int TelnetAppender::getMaxConnections() const
+{
+	return static_cast<int>(_priv->connections.size());
+}
+
+void TelnetAppender::setMaxConnections(int newValue)
+{
+	std::lock_guard<std::recursive_mutex> lock(_priv->mutex);
+	if (_priv->connections.size() < newValue)
+		_priv->connections.resize(newValue);
+	else while (newValue < _priv->connections.size())
+	{
+		auto item = _priv->connections.back();
+		_priv->connections.pop_back();
+		if (item)
+		{
+			item->close();
+			--_priv->activeConnections;
+		}
+	}
+}
+
+bool TelnetAppender::requiresLayout() const
+{
+	return false;
+}
